/*
 * Copyright 2010 Canonical, Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of either or both of the following licenses:
 *
 * 1) the GNU Lesser General Public License version 3, as published by the
 * Free Software Foundation; and/or
 * 2) the GNU Lesser General Public License version 2.1, as published by
 * the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranties of
 * MERCHANTABILITY, SATISFACTORY QUALITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the applicable version of the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of both the GNU Lesser General Public
 * License version 3 and version 2.1 along with this program.  If not, see
 * <http://www.gnu.org/licenses/>
 *
 * Authors:
 *    Cody Russell <crussell@canonical.com>
 */

#ifndef __GROPE_GESTURE_MANAGER_H__
#define __GROPE_GESTURE_MANAGER_H__

#include <gtk/gtk.h>

G_BEGIN_DECLS

#define GROPE_TYPE_GESTURE_MANAGER         (grope_gesture_manager_get_type ())
#define GROPE_GESTURE_MANAGER(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), GROPE_TYPE_GESTURE_MANAGER, GropeGestureManager))
#define GROPE_GESTURE_MANAGER_CLASS(c)     (G_TYPE_CHECK_CLASS_CAST ((c), GROPE_TYPE_GESTURE_MANAGER, GropeGestureManagerClass))
#define GROPE_IS_GESTURE_MANAGER(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), GROPE_TYPE_GESTURE_MANAGER))
#define GROPE_IS_GESTURE_MANAGER_CLASS(c)  (G_TYPE_CHECK_CLASS_TYPE ((c), GROPE_TYPE_GESTURE_MANAGER))
#define GROPE_GESTURE_MANAGER_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GROPE_TYPE_GESTURE_MANAGER, GropeGestureManagerClass))

typedef struct _GropeGestureManager        GropeGestureManager;
typedef struct _GropeGestureManagerClass   GropeGestureManagerClass;
typedef struct _GropeGestureManagerPrivate GropeGestureManagerPrivate;

typedef union  _GropeGestureEvent       GropeGestureEvent;
typedef struct _GropeEventGestureDrag   GropeEventGestureDrag;
typedef struct _GropeEventGesturePinch  GropeEventGesturePinch;
typedef struct _GropeEventGestureRotate GropeEventGestureRotate;

typedef enum {
  GROPE_GESTURE_DRAG,
  GROPE_GESTURE_PINCH,
  GROPE_GESTURE_ROTATE
} GropeGestureType;

struct _GropeEventGestureDrag
{
  GropeGestureType  type;
  guint             id;
  GdkWindow        *window;
  GdkWindow        *root;
  GdkWindow        *child;
  guint32           timestamp;
  gint              fingers;
  gdouble           focus_x;
  gdouble           focus_y;
  gint              delta_x;
  gint              delta_y;
  gdouble           velocity_x;
  gdouble           velocity_y;
  gdouble           position_x;
  gdouble           position_y;
};

struct _GropeEventGesturePinch
{
  GropeGestureType  type;
  guint             id;
  GdkWindow        *window;
  GdkWindow        *root;
  GdkWindow        *child;
  guint32           timestamp;
  guint             fingers;
  gdouble           focus_x;
  gdouble           focus_y;
  gdouble           radius_delta;
  gdouble           radial_velocity;
  gdouble           radius;
};

struct _GropeEventGestureRotate
{
  GropeGestureType  type;
  guint             id;
  GdkWindow        *window;
  GdkWindow        *root;
  GdkWindow        *child;
  guint32           timestamp;
  guint             fingers;
  gdouble           focus_x;
  gdouble           focus_y;
  gdouble           angle_delta;
  gdouble           angular_velocity;
  gdouble           angle;
};

union _GropeGestureEvent
{
  GropeGestureType        type;
  GropeEventGestureDrag   drag;
  GropeEventGesturePinch  pinch;
  GropeEventGestureRotate rotate;
};

struct _GropeGestureManager
{
  GObject parent_instance;

  GropeGestureManagerPrivate *priv;
};

struct _GropeGestureManagerClass
{
  GObjectClass parent_class;
};

typedef void (* GropeGestureCallback) (GtkWindow          *window,
				       GropeGestureEvent  *gesture,
                                       gpointer            user_data);

GType                grope_gesture_manager_get_type        (void) G_GNUC_CONST;
GropeGestureManager *grope_gesture_manager_get             (void);
void                 grope_gesture_manager_register_window (GropeGestureManager *manager,
							    GtkWindow           *window,
							    GropeGestureType     gesture_type,
							    gint                 touch_points,
							    GropeGestureCallback start,
							    GropeGestureCallback update,
							    GropeGestureCallback end,
                                                            gpointer             user_data,
                                                            GDestroyNotify       destroy);

G_END_DECLS

#endif /* __GROPE_GESTURE_MANAGER_H__ */
